@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the Linux console (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2001 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2002 by The BRLTTY Team. All rights reserved.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -24,7 +24,7 @@
  */
 
 #define VERSION \
-"Braille Lite driver, version 0.5.7 (January 2002)"
+"Braille Lite driver, version 0.5.10 (April 2002)"
 
 #define BRL_C
 
@@ -47,8 +47,9 @@
 
 typedef enum {
   PARM_BAUDRATE=0,
+  PARM_KBEMU
 } DriverParameter;
-#define BRLPARMS "baudrate"
+#define BRLPARMS "baudrate", "kbemu"
 
 #include "../brl_driver.h"
 
@@ -73,6 +74,7 @@ static int intoverride = 0;	/* internal override flag -
 				 * highly dubious behaviour ...
 				 */
 static int int_cursor = 0;	/* position of internal cursor: 0 = none */
+static int kbemu = 0; /* keyboard emulation (whether you can type) */
 
 /* The input queue is only manipulated by the qput() and qget()
  * functions.
@@ -145,6 +147,10 @@ brl_initialize (char **parameters, brldim * brl, const char *brldev)
      || !validateBaud(&baudrate, "baud rate",
 		     parameters[PARM_BAUDRATE], good_baudrates))
     baudrate = BAUDRATE;
+  if(*parameters[PARM_KBEMU])
+    validateYesNo(&kbemu, "keyboard emulation initial state",
+		  parameters[PARM_KBEMU]);
+  kbemu = !!kbemu;
 
   /* Open the Braille display device for random access */
   LogPrint(LOG_DEBUG, "Opening serial port %s", brldev);
@@ -377,7 +383,6 @@ brl_writeWindow (brldim * brl)
 static int
 brl_read (DriverCommandContext cmds)
 {
-  static int kbemu = 0;		/* keyboard emulation flag */
   static int state = 0;		/* 0 = normal - transparent
 				 * 1 = positioning internal cursor
 				 * 2 = setting repeat count
@@ -427,12 +432,23 @@ brl_read (DriverCommandContext cmds)
 	  if(key.cmd == CR_CUTRECT || key.cmd == CR_CUTLINE)
 	    key.cmd += blitesz-1;
 
+	  if(key.spcbar && (key.cmd &VAL_BLK_MASK) == VAL_PASSKEY) {
+	    if(!kbemu)
+	      return EOF;
+	    if (!shiftlck)
+	      shift = 0;
+	    ctrl = meta = 0;
+#ifdef USE_TEXTTRANS
+	    dot8shift = 0;
+#endif
+	  }
+
 	  /* always OK if chorded */
 	  if (key.spcbar)
 	    return key.cmd;
 
 	  /* kbemu could be on, then go on */
-	  if (kbemu)
+	  if (kbemu && cmds == CMDS_SCREEN)
 	    break;
 
 	  /* if it's a dangerous command it should have been chorded */
@@ -480,7 +496,7 @@ brl_read (DriverCommandContext cmds)
 	    return CMD_NOOP;
 	  case ' ':		/* practical exception for */
 	    /* If keyboard mode off, space bar == CMD_HOME */
-	    if (!kbemu)
+	    if (!kbemu || cmds != CMDS_SCREEN)
 	      return CMD_HOME;
 	  }
 
@@ -501,13 +517,10 @@ brl_read (DriverCommandContext cmds)
 	  ctrl = 1;
 	  break;
 	case 0x80:		/* dot 8 */
-#ifndef USE_TEXTTRANS
-#if 0
-	  outmsg[0] = 27;
-	  outmsg[1] = 0;
-#endif
-#else
+#ifdef USE_TEXTTRANS
 	  dot8shift = 1;
+#else
+	  meta = 1;
 #endif
 	  break;
 	}
@@ -535,46 +548,6 @@ brl_read (DriverCommandContext cmds)
 	  case BLT_META:	/* meta next */
 	    meta = 1;
 	    return CMD_NOOP;
-	  case BLT_ESCAPE:
-	    if (!shiftlck)
-	      shift = 0;
-	    ctrl = meta = 0;
-#ifdef USE_TEXTTRANS
-	    dot8shift = 0;
-#endif
-	    return VAL_PASSKEY | VPK_ESCAPE;
-	  case BLT_TAB:
-	    if (!shiftlck)
-	      shift = 0;
-	    ctrl = meta = 0;
-#ifdef USE_TEXTTRANS
-	    dot8shift = 0;
-#endif
-	    return VAL_PASSKEY | VPK_TAB;
-	  case BLT_BACKSP:	/* remove backwards */
-	    if (!shiftlck)
-	      shift = 0;
-	    ctrl = meta = 0;
-#ifdef USE_TEXTTRANS
-	    dot8shift = 0;
-#endif
-	    return VAL_PASSKEY | VPK_BACKSPACE;
-	  case BLT_DELETE:	/* remove forward */
-	    if (!shiftlck)
-	      shift = 0;
-	    ctrl = meta = 0;
-#ifdef USE_TEXTTRANS
-	    dot8shift = 0;
-#endif
-	    return VAL_PASSKEY | VPK_DELETE;
-	  case BLT_ENTER:	/* enter - do ^m, or ^j if control-enter */
-	    if (!shiftlck)
-	      shift = 0;
-	    ctrl = meta = 0;
-#ifdef USE_TEXTTRANS
-	    dot8shift = 0;
-#endif
-	    return VAL_PASSKEY | VPK_RETURN;
 	  case BLT_ABORT:	/* abort - quit keyboard emulation */
 	    kbemu = 0;
 	    message ("keyboard emu off", MSG_SILENT);
@@ -728,10 +701,6 @@ brl_read (DriverCommandContext cmds)
       }
       return CMD_NOOP;
     case 3:			/* preferences options */
-#if 0 /* We remove this restriction. */
-      if (!key.spcbar)		/* not chorded */
-	return CMD_NOOP;
-#endif
       switch (key.asc)
 	{
 	case 'm':		/* preferences menu */
