@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the Linux console (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2001 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2002 by The BRLTTY Team. All rights reserved.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -31,6 +31,7 @@
 #include "brl.h"
 #include "spk.h"
 #include "scr.h"
+#include "contract.h"
 #include "tunes.h"
 #include "cut.h"
 #include "misc.h"
@@ -53,7 +54,15 @@ short vwinshift;                /* Window vertical distance */
 Preferences prefs;                /* environment (i.e. global) parameters */
 brldim brl;                        /* For the Braille routines */
 ScreenDescription scr;                        /* For screen state infos */
-unsigned int TickCount = 0;        /* incremented each main loop cycle */
+
+static int contracted = 0;
+static int contractedStart;
+static int contractedLength;
+static int contractedOffsets[0X100];
+static int contractedTrack = 0;
+
+static unsigned char statusCells[StatusCellCount];        /* status cell buffer */
+static unsigned int TickCount = 0;        /* incremented each main loop cycle */
 
 
 /*
@@ -61,21 +70,18 @@ unsigned int TickCount = 0;        /* incremented each main loop cycle */
  */
 
 #define BRL_ISUPPER(c) \
-        (isupper (c) || (c) == '@' || (c) == '[' || (c) == '^' || \
-         (c) == ']' || (c) == '\\')
+  (isupper((c)) || (c)=='@' || (c)=='[' || (c)=='^' || (c)==']' || (c)=='\\')
 
-#define TOGGLEPLAY(var)  playTune((var) ? &tune_toggle_on : &tune_toggle_off)
+#define TOGGLEPLAY(var) playTune((var)? &tune_toggle_on: &tune_toggle_off)
 #define TOGGLE(var) \
-        (var = (keypress & VAL_SWITCHON) \
-               ? 1 : ((keypress & VAL_SWITCHOFF) \
-                      ? 0 : (!var)))
+  (var = (keypress & VAL_SWITCHON)? 1: ((keypress & VAL_SWITCHOFF)? 0: (!var)))
 
 
-unsigned char *curtbl = texttrans;        /* active translation table */
+unsigned char *curtbl = textTable;        /* active translation table */
 
 static void
 setTranslationTable (int attributes) {
-  curtbl = attributes? attribtrans: texttrans;
+  curtbl = attributes? attributesTable: textTable;
 }
 
 
@@ -132,108 +138,113 @@ switchto (unsigned int scrno) {
 
 void
 clearStatusCells (void) {
-   memset(statcells, 0, sizeof(statcells));
-   braille->writeStatus(statcells);
+   memset(statusCells, 0, sizeof(statusCells));
+   braille->writeStatus(statusCells);
 }
 
 static void
 setStatusCells (void) {
-   memset(statcells, 0, sizeof(statcells));
+   memset(statusCells, 0, sizeof(statusCells));
    switch (prefs.stcellstyle) {
       case ST_AlvaStyle:
          if ((dispmd & HELP_SCRN) == HELP_SCRN) {
-            statcells[0] = texttrans['h'];
-            statcells[1] = texttrans['l'];
-            statcells[2] = texttrans['p'];
+            statusCells[0] = textTable['h'];
+            statusCells[1] = textTable['l'];
+            statusCells[2] = textTable['p'];
          } else {
             /* The coords are given with letters as the DOS tsr */
-            statcells[0] = ((TickCount / 16) % (scr.posy / 25 + 1))? 0:
-                           texttrans[scr.posy % 25 + 'a'] |
-                           ((scr.posx / brl.x) << 6);
-            statcells[1] = ((TickCount / 16) % (p->winy / 25 + 1))? 0:
-                           texttrans[p->winy % 25 + 'a'] |
-                           ((p->winx / brl.x) << 6);
-            statcells[2] = texttrans[(p->showAttributes)? 'a':
-                                      ((dispmd & FROZ_SCRN) == FROZ_SCRN)? 'f':
-                                      p->trackCursor? 't':
-                                      ' '];
+            statusCells[0] = ((TickCount / 16) % (scr.posy / 25 + 1))? 0:
+                             textTable[scr.posy % 25 + 'a'] |
+                             ((scr.posx / brl.x) << 6);
+            statusCells[1] = ((TickCount / 16) % (p->winy / 25 + 1))? 0:
+                             textTable[p->winy % 25 + 'a'] |
+                             ((p->winx / brl.x) << 6);
+            statusCells[2] = textTable[(p->showAttributes)? 'a':
+                                        ((dispmd & FROZ_SCRN) == FROZ_SCRN)? 'f':
+                                        p->trackCursor? 't':
+                                        ' '];
          }
          break;
       case ST_TiemanStyle:
-         statcells[0] = (portraitDigits[(p->winx / 10) % 10] << 4) |
-                        portraitDigits[(scr.posx / 10) % 10];
-         statcells[1] = (portraitDigits[p->winx % 10] << 4) |
-                        portraitDigits[scr.posx % 10];
-         statcells[2] = (portraitDigits[(p->winy / 10) % 10] << 4) |
-                        portraitDigits[(scr.posy / 10) % 10];
-         statcells[3] = (portraitDigits[p->winy % 10] << 4) |
-                        portraitDigits[scr.posy % 10];
-         statcells[4] = ((dispmd & FROZ_SCRN) == FROZ_SCRN? 1: 0) |
-                        (prefs.csrvis << 1) |
-                        (p->showAttributes << 2) |
-                        (prefs.csrsize << 3) |
-                        (prefs.sound << 4) |
-                        (prefs.csrblink << 5) |
-                        (p->trackCursor << 6) |
-                        (prefs.slidewin << 7);
+         statusCells[0] = (portraitDigits[(p->winx / 10) % 10] << 4) |
+                          portraitDigits[(scr.posx / 10) % 10];
+         statusCells[1] = (portraitDigits[p->winx % 10] << 4) |
+                          portraitDigits[scr.posx % 10];
+         statusCells[2] = (portraitDigits[(p->winy / 10) % 10] << 4) |
+                          portraitDigits[(scr.posy / 10) % 10];
+         statusCells[3] = (portraitDigits[p->winy % 10] << 4) |
+                          portraitDigits[scr.posy % 10];
+         statusCells[4] = ((dispmd & FROZ_SCRN) == FROZ_SCRN? 1: 0) |
+                          (prefs.csrvis << 1) |
+                          (p->showAttributes << 2) |
+                          (prefs.csrsize << 3) |
+                          (prefs.tunes << 4) |
+                          (prefs.csrblink << 5) |
+                          (p->trackCursor << 6) |
+                          (prefs.slidewin << 7);
          break;
       case ST_PB80Style:
-         statcells[0] = (portraitDigits[(p->winy+1) % 10] << 4) |
-                        portraitDigits[((p->winy+1) / 10) % 10];
+         statusCells[0] = (portraitDigits[(p->winy+1) % 10] << 4) |
+                          portraitDigits[((p->winy+1) / 10) % 10];
          break;
       case ST_Generic:
-         statcells[FirstStatusCell] = FSC_GENERIC;
-         statcells[STAT_BrlCol] = p->winx+1;
-         statcells[STAT_BrlRow] = p->winy+1;
-         statcells[STAT_CsrCol] = scr.posx+1;
-         statcells[STAT_CsrRow] = scr.posy+1;
-         statcells[STAT_ScrNum] = scr.no;
-         statcells[STAT_TrkCsr] = p->trackCursor;
-         statcells[STAT_AttrDisp] = p->showAttributes;
-         statcells[STAT_FrzScr] = (dispmd & FROZ_SCRN) == FROZ_SCRN;
-         statcells[STAT_VisCsr] = prefs.csrvis;
-         statcells[STAT_BlkCsr] = prefs.csrsize;
-         statcells[STAT_BlnkCsr] = prefs.csrblink;
-         statcells[STAT_BlnkCaps] = prefs.capblink;
-         statcells[STAT_SixDot] = prefs.sixdots;
-         statcells[STAT_AlertTunes] = prefs.sound;
-         statcells[STAT_IdentLines] = prefs.skpidlns;
-         statcells[STAT_VisAttr] = prefs.attrvis;
-         statcells[STAT_BlnkAttr] = prefs.attrblink;
+         statusCells[FirstStatusCell] = FSC_GENERIC;
+         statusCells[STAT_BRLCOL] = p->winx+1;
+         statusCells[STAT_BRLROW] = p->winy+1;
+         statusCells[STAT_CSRCOL] = scr.posx+1;
+         statusCells[STAT_CSRROW] = scr.posy+1;
+         statusCells[STAT_SCRNUM] = scr.no;
+         statusCells[STAT_FREEZE] = (dispmd & FROZ_SCRN) == FROZ_SCRN;
+         statusCells[STAT_DISPMD] = p->showAttributes;
+         statusCells[STAT_SIXDOTS] = prefs.sixdots;
+         statusCells[STAT_SLIDEWIN] = prefs.slidewin;
+         statusCells[STAT_SKPIDLNS] = prefs.skpidlns;
+         statusCells[STAT_SKPBLNKWINS] = prefs.skpblnkwins;
+         statusCells[STAT_CSRVIS] = prefs.csrvis;
+         statusCells[STAT_CSRHIDE] = p->hideCursor;
+         statusCells[STAT_CSRTRK] = p->trackCursor;
+         statusCells[STAT_CSRSIZE] = prefs.csrsize;
+         statusCells[STAT_CSRBLINK] = prefs.csrblink;
+         statusCells[STAT_ATTRVIS] = prefs.attrvis;
+         statusCells[STAT_ATTRBLINK] = prefs.attrblink;
+         statusCells[STAT_CAPBLINK] = prefs.capblink;
+         statusCells[STAT_TUNES] = prefs.tunes;
+         statusCells[STAT_HELP] = (dispmd & HELP_SCRN) != 0;
+         statusCells[STAT_INFO] = infmode;
          break;
       case ST_MDVStyle:
-         statcells[0] = portraitDigits[((p->winy+1) / 10) % 10] |
-                        (portraitDigits[((p->winx+1) / 10) % 10] << 4);
-         statcells[1] = portraitDigits[(p->winy+1) % 10] |
-                        (portraitDigits[(p->winx+1) % 10] << 4);
+         statusCells[0] = portraitDigits[((p->winy+1) / 10) % 10] |
+                          (portraitDigits[((p->winx+1) / 10) % 10] << 4);
+         statusCells[1] = portraitDigits[(p->winy+1) % 10] |
+                          (portraitDigits[(p->winx+1) % 10] << 4);
          break;
       case ST_VoyagerStyle: /* 3cells (+1 blank) */
-         statcells[0] = (portraitDigits[p->winy % 10] << 4) |
-                        portraitDigits[(p->winy / 10) % 10];
-         statcells[1] = (portraitDigits[scr.posy % 10] << 4) |
-                        portraitDigits[(scr.posy / 10) % 10];
+         statusCells[0] = (portraitDigits[p->winy % 10] << 4) |
+                          portraitDigits[(p->winy / 10) % 10];
+         statusCells[1] = (portraitDigits[scr.posy % 10] << 4) |
+                          portraitDigits[(scr.posy / 10) % 10];
          if ((dispmd & FROZ_SCRN) == FROZ_SCRN)
-            statcells[2] = texttrans['F'];
+            statusCells[2] = textTable['F'];
          else
-            statcells[2] = (portraitDigits[scr.posx % 10] << 4) |
-                           portraitDigits[(scr.posx / 10) % 10];
+            statusCells[2] = (portraitDigits[scr.posx % 10] << 4) |
+                             portraitDigits[(scr.posx / 10) % 10];
          break;
       default:
          break;
    }
-   braille->writeStatus(statcells);
+   braille->writeStatus(statusCells);
 }
 
 void
 setStatusText (const unsigned char *text) {
    int i;
-   memset(statcells, 0, sizeof(statcells));
-   for (i=0; i<sizeof(statcells); ++i) {
+   memset(statusCells, 0, sizeof(statusCells));
+   for (i=0; i<sizeof(statusCells); ++i) {
       unsigned char character = text[i];
       if (!character) break;
-      statcells[i] = texttrans[character];
+      statusCells[i] = textTable[character];
    }
-   braille->writeStatus(statcells);
+   braille->writeStatus(statusCells);
 }
 
 static void
@@ -273,19 +284,19 @@ showInfo (void)
                 portraitDigits[(scr.posy / 10) % 10];
     infbuf[3] = portraitDigits[p->winy % 10] << 4 | portraitDigits[scr.posy % 10];
     infbuf[4] = (((dispmd & FROZ_SCRN) == FROZ_SCRN)? B1: 0) |
-                (p->showAttributes?    B2: 0) |
-                (prefs.sound?    B3: 0) |
-                (prefs.csrvis?   B4: 0) |
-                (prefs.csrsize?  B5: 0) |
-                (prefs.csrblink? B6: 0) |
-                (p->trackCursor?      B7: 0) |
-                (prefs.slidewin? B8: 0);
+                (p->showAttributes? B2: 0) |
+                (prefs.tunes?       B3: 0) |
+                (prefs.csrvis?      B4: 0) |
+                (prefs.csrsize?     B5: 0) |
+                (prefs.csrblink?    B6: 0) |
+                (p->trackCursor?    B7: 0) |
+                (prefs.slidewin?    B8: 0);
 
     /* We have to do the Braille translation ourselves, since
      * we don't want the first five characters translated ...
      */
     for (i=5; infbuf[i]; i++)
-      infbuf[i] = texttrans[infbuf[i]];
+      infbuf[i] = textTable[infbuf[i]];
 
     memcpy(brl.disp, infbuf, brl.x*brl.y);
     braille->writeWindow(&brl);
@@ -372,20 +383,42 @@ placeWindowHorizontally (int x) {
 
 static void 
 trackCursor (int place) {
-  if (prefs.slidewin) {
-    int reset = brl.x * 3 / 10;
-    int trigger = prefs.eager_slidewin? brl.x*3/20: 0;
-    if (place)
-      if ((scr.posx < p->winx) || (scr.posx >= (p->winx + brl.x)) ||
-          (scr.posy < p->winy) || (scr.posy >= (p->winy + brl.y)))
-        placeWindowHorizontally(scr.posx);
-    if (scr.posx < (p->winx + trigger))
-      p->winx = MAX(scr.posx-reset, 0);
-    else if (scr.posx >= (p->winx + brl.x - trigger))
-      p->winx = MIN(scr.posx+reset+1, scr.cols) - brl.x;
-  } else
-    placeWindowHorizontally(scr.posx);
-  slideWindowVertically(scr.posy);
+  if (contracted) {
+    p->winy = scr.posy;
+    if (scr.posx < p->winx) {
+      int length = scr.posx + 1;
+      unsigned char buffer[length];
+      int onspace = 1;
+      readScreen((ScreenBox){0, p->winy, length, 1}, buffer, SCR_TEXT);
+      while (length) {
+        if ((isspace(buffer[--length]) != 0) != onspace) {
+          if (onspace) {
+            onspace = 0;
+          } else {
+            ++length;
+            break;
+          }
+        }
+      }
+      p->winx = length;
+    }
+    contractedTrack = 1;
+  } else {
+    if (prefs.slidewin) {
+      int reset = brl.x * 3 / 10;
+      int trigger = prefs.eager_slidewin? brl.x*3/20: 0;
+      if (place)
+        if ((scr.posx < p->winx) || (scr.posx >= (p->winx + brl.x)) ||
+            (scr.posy < p->winy) || (scr.posy >= (p->winy + brl.y)))
+          placeWindowHorizontally(scr.posx);
+      if (scr.posx < (p->winx + trigger))
+        p->winx = MAX(scr.posx-reset, 0);
+      else if (scr.posx >= (p->winx + brl.x - trigger))
+        p->winx = MAX(MIN(scr.posx+reset+1, scr.cols)-brl.x, 0);
+    } else
+      placeWindowHorizontally(scr.posx);
+    slideWindowVertically(scr.posy);
+  }
 }
 
 static int speaking_start_line = 0;
@@ -569,6 +602,26 @@ testPrompt (int column, int row, void *data) {
   return memcmp(buffer, prompt, count) == 0;
 }
 
+static int
+getOffset (int arg, int end) {
+  if (contracted) {
+    int result = 0;
+    int index;
+    for (index=0; index<contractedLength; ++index) {
+      int offset = contractedOffsets[index];
+      if (offset != -1) {
+        if (offset > arg) {
+          if (end) result = index - 1;
+          break;
+        }
+        result = index;
+      }
+    }
+    return result;
+  }
+  return arg;
+}
+
 int
 main (int argc, char *argv[]) {
   int keypress;                        /* character received from braille display */
@@ -580,7 +633,8 @@ main (int argc, char *argv[]) {
   short attron = 0;
   short attrcntr = 1;
   short oldwinx, oldwiny;
-  short speaking_scrno = -1, speaking_prev_inx = -1;
+  short speaking_scrno = -1, speaking_prev_inx = -1,
+    speak_dont_track = 0;
 
 #ifdef INIT_PATH
   if (getpid() == 1) {
@@ -599,7 +653,7 @@ main (int argc, char *argv[]) {
         fflush(stderr);
         exit(1);
       case 0: { /* child */
-        static char *arguments[] = {"brltty", "-E", "-linfo", "-v", NULL};
+        static char *arguments[] = {"brltty", "-E", "-n", "-e", "-linfo", NULL};
         argv = arguments;
         argc = (sizeof(arguments) / sizeof(arguments[0])) - 1;
         break;
@@ -675,7 +729,13 @@ main (int argc, char *argv[]) {
       if (!executeScreenCommand(keypress)) {
         switch (keypress & ~VAL_SWITCHMASK) {
           case CMD_NOOP:        /* do nothing but loop */
-            continue;
+            if (keypress & VAL_SWITCHON)
+              playTune(&tune_toggle_on);
+            else if (keypress & VAL_SWITCHOFF)
+              playTune(&tune_toggle_off);
+            else
+              continue;
+            break;
           case CMD_RESTARTBRL:
             stopBrailleDriver();
             playTune(&tune_braille_off);
@@ -817,8 +877,9 @@ main (int argc, char *argv[]) {
           case CMD_FWINRT:
             if (!(prefs.skpblnkwins && (prefs.skpblnkwinsmode == sbwAll))) {
               int oldX = p->winx;
-              if (p->winx < (scr.cols - brl.x)) {
-                p->winx = MIN(p->winx+fwinshift, scr.cols-offr);
+              int rwinshift = contracted? contractedLength: fwinshift;
+              if (p->winx < (scr.cols - rwinshift)) {
+                p->winx += rwinshift;
                 if (prefs.skpblnkwins) {
                   if (!prefs.csrvis ||
                       (scr.posy != p->winy) ||
@@ -856,8 +917,9 @@ main (int argc, char *argv[]) {
             int charIndex;
             char buffer[scr.cols];
             while (1) {
-              if (p->winx < (scr.cols - brl.x)) {
-                p->winx = MIN(p->winx+fwinshift, scr.cols-offr);
+              int rwinshift = contracted? contractedLength: fwinshift;
+              if (p->winx < (scr.cols - rwinshift)) {
+                p->winx += rwinshift;
               } else {
                 if (p->winy >= (scr.rows - brl.y)) {
                   playTune(&tune_bounce);
@@ -1100,7 +1162,7 @@ main (int argc, char *argv[]) {
             playTune(&tune_bad_command);
             break;
           case CMD_TUNES:
-            TOGGLEPLAY ( TOGGLE(prefs.sound) );        /* toggle sound on/off */
+            TOGGLEPLAY ( TOGGLE(prefs.tunes) );        /* toggle sound on/off */
             break;
           case CMD_DISPMD:
             setTranslationTable(TOGGLE(p->showAttributes));
@@ -1215,16 +1277,13 @@ main (int argc, char *argv[]) {
                            buffer+i, SCR_ATTRIB);
                 speech->express(buffer, i);
               }else speech->say(buffer, i);
-              if ((keypress & ~VAL_SWITCHMASK) == CMD_SAYALL) {
-                speaking_scrno = scr.no;
-                speaking_start_line = p->winy;
-              } else {
-                /* 
-                 * We don't want speech tracking to move the braille window 
-                 * for a single line reading.
-                 */ 
-                speaking_scrno = -1;
-              }
+	      speaking_scrno = scr.no;
+	      speaking_start_line = p->winy;
+	      /* 
+	       * We don't want speech tracking to move the braille window 
+	       * for a single line reading.
+	       */ 
+              speak_dont_track = ((keypress & ~VAL_SWITCHMASK) == CMD_SAY);
             }
             break;
           case CMD_MUTE:
@@ -1307,38 +1366,46 @@ main (int argc, char *argv[]) {
                 }
                 break;
               case VAL_PASSDOTS:
-                if (!insertCharacter(untexttrans[arg], flags)) {
+                if (!insertCharacter(untextTable[arg], flags)) {
                   playTune(&tune_bad_command);
                 }
                 break;
               case CR_ROUTE:
-                if (arg < brl.x)
+                if (arg < brl.x) {
+                  arg = getOffset(arg, 0);
                   if (routeCursor(MIN(p->winx+arg, scr.cols-1), p->winy, curscr))
                     break;
+                }
                 playTune(&tune_bad_command);
                 break;
               case CR_CUTBEGIN:
-                if (arg < brl.x && p->winx+arg < scr.cols)
+                if (arg < brl.x && p->winx+arg < scr.cols) {
+                  arg = getOffset(arg, 0);
                   cut_begin(p->winx+arg, p->winy);
-                else
+                } else
                   playTune(&tune_bad_command);
                 break;
               case CR_CUTAPPEND:
-                if (arg < brl.x && p->winx+arg < scr.cols)
+                if (arg < brl.x && p->winx+arg < scr.cols) {
+                  arg = getOffset(arg, 0);
                   cut_append(p->winx+arg, p->winy);
-                else
+                } else
                   playTune(&tune_bad_command);
                 break;
               case CR_CUTRECT:
-                if (arg < brl.x && p->winx+arg < scr.cols)
+                if (arg < brl.x) {
+                  arg = getOffset(arg, 1);
                   if (cut_rectangle(MIN(p->winx+arg, scr.cols-1), p->winy))
                     break;
+                }
                 playTune(&tune_bad_command);
                 break;
               case CR_CUTLINE:
-                if (arg < brl.x && p->winx+arg < scr.cols)
+                if (arg < brl.x) {
+                  arg = getOffset(arg, 1);
                   if (cut_line(MIN(p->winx+arg, scr.cols-1), p->winy))
                     break;
+                }
                 playTune(&tune_bad_command);
                 break;
               case CR_DESCCHAR:
@@ -1349,6 +1416,7 @@ main (int argc, char *argv[]) {
                   };
                   char buffer[0X40];
                   unsigned char character, attributes;
+                  arg = getOffset(arg, 0);
                   readScreen((ScreenBox){p->winx+arg, p->winy, 1, 1},
                              &character, SCR_TEXT);
                   readScreen((ScreenBox){p->winx+arg, p->winy, 1, 1},
@@ -1364,9 +1432,10 @@ main (int argc, char *argv[]) {
                   playTune(&tune_bad_command);
                 break;
               case CR_SETLEFT:
-                if (arg < brl.x && p->winx+arg < scr.cols)
+                if (arg < brl.x && p->winx+arg < scr.cols) {
+                  arg = getOffset(arg, 0);
                   p->winx += arg;
-                else
+                } else
                   playTune(&tune_bad_command);
                 break;
               case CR_SETMARK: {
@@ -1394,6 +1463,7 @@ main (int argc, char *argv[]) {
               case CR_NXINDENT:
                 increment = 1;
               findIndent:
+                arg = getOffset(arg, 0);
                 findRow(MIN(p->winx+arg, scr.cols-1),
                         increment, testIndent, NULL);
                 break;
@@ -1407,11 +1477,12 @@ main (int argc, char *argv[]) {
         }
       }
 
-      if ((p->winx != oldmotx) || (p->winy != oldmoty))
-        { // The window has been manually moved.
-          p->motx = p->winx;
-          p->moty = p->winy;
-        }
+      if ((p->winx != oldmotx) || (p->winy != oldmoty)) {
+        // The window has been manually moved.
+        p->motx = p->winx;
+        p->moty = p->winy;
+        contracted = 0;
+      }
     }
         
     /*
@@ -1459,31 +1530,26 @@ main (int argc, char *argv[]) {
     /* speech tracking */
     speech->doTrack(); /* called continually even if we're not tracking
                              so that the pipe doesn't fill up. */
-    if(p->trackCursor && speech->isSpeaking() && scr.no == speaking_scrno) {
-      int inx = speech->getTrack();
-      if(inx != speaking_prev_inx) {
-        trackSpeech(speaking_prev_inx = inx);
-      }
-    }
-    else /* cursor tracking */
-    if (p->trackCursor)
-      {
+    if (p->trackCursor) {
+      if (speech->isSpeaking() && scr.no == speaking_scrno
+	  && !speak_dont_track) {
+        int inx = speech->getTrack();
+        if (inx != speaking_prev_inx) {
+          trackSpeech(speaking_prev_inx = inx);
+        }
+      } else {
         /* If cursor moves while blinking is on */
-        if (prefs.csrblink)
-          {
-            if (scr.posy != p->trky)
-              {
-                /* turn off cursor to see what's under it while changing lines */
-                csron = 0;
-                csrcntr = prefs.csroffcnt;
-              }
-            else if (scr.posx != p->trkx)
-              {
-                /* turn on cursor to see it moving on the line */
-                csron = 1;
-                csrcntr = prefs.csroncnt;
-              }
+        if (prefs.csrblink) {
+          if (scr.posy != p->trky) {
+            /* turn off cursor to see what's under it while changing lines */
+            csron = 0;
+            csrcntr = prefs.csroffcnt;
+          } else if (scr.posx != p->trkx) {
+            /* turn on cursor to see it moving on the line */
+            csron = 1;
+            csrcntr = prefs.csroncnt;
           }
+        }
         /* If the cursor moves in cursor tracking mode: */
         if (!csr_active && (scr.posx != p->trkx || scr.posy != p->trky)) {
           trackCursor(0);
@@ -1491,12 +1557,14 @@ main (int argc, char *argv[]) {
           p->trky = scr.posy;
         }
       }
+    }
+
     /* If attribute underlining is blinking during display movement */
-    if(prefs.attrvis && prefs.attrblink){
+    if (prefs.attrvis && prefs.attrblink) {
       /* We could check to see if we changed screen, but that doesn't
          really matter... this is mainly for when you are hunting up/down
          for the line with attributes. */
-      if(p->winx != oldwinx || p->winy != oldwiny){
+      if (p->winx != oldwinx || p->winy != oldwiny) {
         attron = 1;
         attrcntr = prefs.attroncnt;
       }
@@ -1504,76 +1572,157 @@ main (int argc, char *argv[]) {
          stationnary and the attributes themselves are moving
          (example: tin). */
     }
+
     oldwinx = p->winx; oldwiny = p->winy;
-    /* If not in info mode, get screen image: */
     if (infmode) {
       showInfo();
     } else {
-      int winlen = MIN(brl.x, scr.cols -p->winx);
-      readScreen((ScreenBox){p->winx, p->winy, winlen, brl.y},
-                 brl.disp,
-                       p->showAttributes? SCR_ATTRIB: SCR_TEXT);
-      if (prefs.capblink && !capon)
-        for (i = 0; i < winlen * brl.y; i++)
-          if (BRL_ISUPPER (brl.disp[i]))
-            brl.disp[i] = ' ';
+      int cursorLocation = -1;
+      contracted = 0;
+      if (prefs.sixdots && contractionTable) {
+        int windowLength = brl.x * brl.y;
+        while (1) {
+          int cursorOffset = cursorLocation;
+          int inputLength = scr.cols - p->winx;
+          int outputLength = windowLength;
+          unsigned char inputBuffer[inputLength];
+          unsigned char outputBuffer[outputLength];
 
-      /*
-       * Do Braille translation using current table: 
-       */
-      if (prefs.sixdots && curtbl != attribtrans)
-        for (i = 0; i < winlen * brl.y; brl.disp[i] = \
-        curtbl[brl.disp[i]] & 0x3f, i++);
-      else
-        for (i = 0; i < winlen * brl.y; brl.disp[i] = \
-        curtbl[brl.disp[i]], i++);
+          if ((scr.posy == p->winy) && (scr.posx >= p->winx)) cursorOffset = scr.posx - p->winx;
+          readScreen((ScreenBox){p->winx, p->winy, inputLength, 1}, inputBuffer, SCR_TEXT);
+          for (i=0; i<inputLength; ++i) contractedOffsets[i] = -1;
+          if (!contractText(contractionTable,
+                            inputBuffer, &inputLength,
+                            outputBuffer, &outputLength,
+                            contractedOffsets, cursorOffset))
+            break;
 
-      if(winlen <brl.x) {
-        /* We got a rectangular piece of text with getscr. But the display
-           is in an off-right position, with some cells at the end blank.
-           So we'll insert these cells and blank them. */
-        for(i=brl.y-1; i>0; i--)
-          memmove(brl.disp +i*brl.x, brl.disp +i*winlen, winlen);
-        for(i=0; i<brl.y; i++)
-          memset(brl.disp +i*brl.x +winlen, 0, brl.x-winlen);
+          if (contractedTrack) {
+            int inputEnd = inputLength;
+            if (outputLength == windowLength) {
+              int inputIndex = inputEnd;
+              while (inputIndex) {
+                int offset = contractedOffsets[--inputIndex];
+                if (offset != -1) {
+                  if (offset != outputLength) break;
+                  inputEnd = inputIndex;
+                }
+              }
+            }
+            if (scr.posx >= (p->winx + inputEnd)) {
+              int offset = 0;
+              int onspace = 0;
+              int length = scr.cols - p->winx;
+              unsigned char buffer[length];
+              readScreen((ScreenBox){p->winx, p->winy, length, 1}, buffer, SCR_TEXT);
+              while (offset < length) {
+                if ((isspace(buffer[offset]) != 0) != onspace) {
+                  if (onspace) break;
+                  onspace = 1;
+                }
+                ++offset;
+              }
+              if ((offset += p->winx) > scr.posx)
+                p->winx = (p->winx + scr.posx) / 2;
+              else
+                p->winx = offset;
+              continue;
+            }
+          }
+
+          memcpy(brl.disp, outputBuffer, outputLength);
+          memset(brl.disp+outputLength, 0, windowLength-outputLength);
+          while (cursorOffset >= 0) {
+            int offset = contractedOffsets[cursorOffset];
+            if (offset >= 0) {
+              cursorLocation = offset;
+              break;
+            }
+            --cursorOffset;
+          }
+          contractedStart = p->winx;
+          contractedLength = inputLength;
+          contractedTrack = 0;
+          contracted = 1;
+          break;
+        }
       }
-
-      /* Attribute underlining: if viewing text (not attributes), attribute
-         underlining is active and visible and we're not in help, then we
-         get the attributes for the current region and OR the underline. */
-      if (!p->showAttributes && prefs.attrvis && (!prefs.attrblink || attron)) {
-        int x,y;
-        unsigned char attrbuf[winlen*brl.y];
+      if (!contracted) {
+        int winlen = MIN(brl.x, scr.cols-p->winx);
         readScreen((ScreenBox){p->winx, p->winy, winlen, brl.y},
-                   attrbuf, SCR_ATTRIB);
-        for(y=0; y<brl.y; y++)
-          for (x=0; x<winlen; x++)
-            switch (attrbuf[y*winlen + x]) {
-              /* Experimental! Attribute values are hardcoded... */
-              case 0x08: /* dark-gray on black */
-              case 0x07: /* light-gray on black */
-              case 0x17: /* light-gray on blue */
-              case 0x30: /* black on cyan */
-                break;
-              case 0x70: /* black on light-gray */
-                brl.disp[y*brl.x +x] |= ATTR1CHAR;
-                break;
-              case 0x0F: /* white on black */
-              default:
-                brl.disp[y*brl.x + x] |= ATTR2CHAR;
-                break;
-            };
-      }
+                   brl.disp,
+                   p->showAttributes? SCR_ATTRIB: SCR_TEXT);
 
-      /*
-       * If the cursor is visible and in range, and help is off: 
-       */
-      if (prefs.csrvis && !p->hideCursor
-          && (!prefs.csrblink || csron) && scr.posx >= p->winx &&
-          scr.posx < p->winx + brl.x && scr.posy >= p->winy &&
-          scr.posy < p->winy + brl.y)
-        brl.disp[(scr.posy - p->winy) * brl.x + scr.posx - p->winx] |= \
-          prefs.csrsize ? BIG_CSRCHAR : SMALL_CSRCHAR;
+        /* blank out capital letters if they're blinking and should be off */
+        if (prefs.capblink && !capon)
+          for (i=0; i<winlen*brl.y; i++)
+            if (BRL_ISUPPER(brl.disp[i]))
+              brl.disp[i] = ' ';
+
+        /* convert to dots using the current translation table */
+        if ((curtbl == attributesTable) || !prefs.sixdots) {
+          for (
+            i = 0;
+            i < (winlen * brl.y);
+            brl.disp[i] = curtbl[brl.disp[i]], i++
+          );
+        } else {
+          for (
+            i = 0;
+            i < (winlen * brl.y);
+            brl.disp[i] = curtbl[brl.disp[i]] & (B1 | B2 | B3 | B4 | B5 | B6), i++
+          );
+        }
+
+        if (winlen < brl.x) {
+          /* We got a rectangular piece of text with readScreen but the display
+             is in an off-right position with some cells at the end blank
+             so we'll insert these cells and blank them. */
+          for (i=brl.y-1; i>0; i--)
+            memmove(brl.disp+i*brl.x, brl.disp+i*winlen, winlen);
+          for (i=0; i<brl.y; i++)
+            memset(brl.disp+i*brl.x+winlen, 0, brl.x-winlen);
+        }
+
+        /* Attribute underlining: if viewing text (not attributes), attribute
+           underlining is active and visible and we're not in help, then we
+           get the attributes for the current region and OR the underline. */
+        if (!p->showAttributes && prefs.attrvis && (!prefs.attrblink || attron)) {
+          int x,y;
+          unsigned char attrbuf[winlen*brl.y];
+          readScreen((ScreenBox){p->winx, p->winy, winlen, brl.y},
+                     attrbuf, SCR_ATTRIB);
+          for(y=0; y<brl.y; y++)
+            for (x=0; x<winlen; x++)
+              switch (attrbuf[y*winlen + x]) {
+                /* Experimental! Attribute values are hardcoded... */
+                case 0x08: /* dark-gray on black */
+                case 0x07: /* light-gray on black */
+                case 0x17: /* light-gray on blue */
+                case 0x30: /* black on cyan */
+                  break;
+                case 0x70: /* black on light-gray */
+                  brl.disp[y*brl.x +x] |= ATTR1CHAR;
+                  break;
+                case 0x0F: /* white on black */
+                default:
+                  brl.disp[y*brl.x + x] |= ATTR2CHAR;
+                  break;
+              };
+        }
+
+        /*
+         * If the cursor is visible and in range, and help is off: 
+         */
+        if ((scr.posx >= p->winx) && (scr.posx < (p->winx + brl.x)) &&
+            (scr.posy >= p->winy) && (scr.posy < (p->winy + brl.y)))
+          cursorLocation = (scr.posy - p->winy) * brl.x + scr.posx - p->winx;
+      }
+      if (cursorLocation >= 0) {
+        if (prefs.csrvis && !p->hideCursor && (!prefs.csrblink || csron)) {
+          brl.disp[cursorLocation] |= prefs.csrsize? BIG_CSRCHAR: SMALL_CSRCHAR;
+        }
+      }
 
       setStatusCells();
       braille->writeWindow(&brl);
@@ -1589,7 +1738,7 @@ void
 message (const unsigned char *text, short flags) {
    int length = strlen(text);
 
-   if (prefs.sound && !(flags & MSG_SILENT)) {
+   if (prefs.tunes && !(flags & MSG_SILENT)) {
       speech->mute();
       speech->say(text, length);
    }
@@ -1622,7 +1771,7 @@ message (const unsigned char *text, short flags) {
           * ignored, since case can be important, and * blinking caps won't 
           * work ... 
           */
-         for (index=0; index<brl.x*brl.y; ++index) brl.disp[index] = texttrans[brl.disp[index]];
+         for (index=0; index<brl.x*brl.y; ++index) brl.disp[index] = textTable[brl.disp[index]];
          braille->writeWindow(&brl);
 
          if (flags & MSG_WAITKEY)
@@ -1640,9 +1789,9 @@ message (const unsigned char *text, short flags) {
 
 void
 showDotPattern (unsigned char dots, unsigned char duration) {
-  memset(statcells, dots, sizeof(statcells));
+  memset(statusCells, dots, sizeof(statusCells));
   memset(brl.disp, dots, brl.x*brl.y);
-  braille->writeStatus(statcells);
+  braille->writeStatus(statusCells);
   braille->writeWindow(&brl);
   shortdelay(duration);
 }
